Skip to contents

This class is used to represent the decision variables used in optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> Decision

Methods

Public methods

Inherited methods


Decision$clone()

The objects of this class are cloneable with this method.

Usage

Decision$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.