Skip to contents

This class is used to represent approaches for multi-objective optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> MultiObjApproach

Methods

Inherited methods


MultiObjApproach$calculate()

Perform computations that need to be completed before applying the object.

Usage

MultiObjApproach$calculate(x, y)

Arguments

x

list containing a compiled multi-objective optimization problem (e.g., generated with internal function multi_compile()).

y

multi_problem() object.

Returns

Invisible TRUE.


MultiObjApproach$run()

Solve a multi-objective optimization problem to generate a solution.

Usage

MultiObjApproach$run(x)

Arguments

x

list containing a compiled multi-objective optimization problem (e.g., generated with internal function multi_compile()).

Returns

list of solutions.


MultiObjApproach$clone()

The objects of this class are cloneable with this method.

Usage

MultiObjApproach$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.