Skip to contents

This class is used to represent the objective function used in optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> Objective

Public fields

has_targets

logical value indicating if the objective uses targets.

has_weights

logical value indicating if the objective uses weights.

Methods

Inherited methods


Objective$feature_phylogeny()

Obtain the feature phylogeny.

Usage

Objective$feature_phylogeny()

Returns

A ape::phylo() phylogenetic tree object.


Objective$default_feature_weights()

Obtain default feature weights.

Usage

Objective$default_feature_weights()

Returns

A numeric vector with the default feature weights.


Objective$replace_feature_weights()

Should default feature weights be replaced or multiplied by the new weights?

Usage

Objective$replace_feature_weights()

Returns

A logical value.ks


Objective$evaluate()

Calculate the objective value for a solution.

Usage

Objective$evaluate(y, solution)

Arguments

y

ProjectProblem object.

solution

tibble::tibble() object with solution.

Returns

A numeric value.


Objective$clone()

The objects of this class are cloneable with this method.

Usage

Objective$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.