Skip to contents

This class is used to represent targets for optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> Weight

Methods

Inherited methods


Weight$output()

Output the targets.

Usage

Weight$output()

Returns

A numeric matrix.


Weight$clone()

The objects of this class are cloneable with this method.

Usage

Weight$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.