Phylogenetic trees depict the evolutionary relationships between different species. Each branch in a phylogenetic tree represents a period of evolutionary history. Species that are connected to the same branch share the same period of evolutionary history represented by the branch. This function creates a matrix that shows which species are connected with which branches. In other words, it creates a matrix that shows which periods of evolutionary history each species has experienced.
Usage
branch_matrix(x, ...)
# Default S3 method
branch_matrix(x, ...)
# S3 method for class 'phylo'
branch_matrix(x, assert_validity = TRUE, ...)Arguments
- x
ape::phylo()tree object.- ...
not used.
- assert_validity
logicalvalue (i.e.,TRUEorFALSEindicating if the argument toxshould be checked for validity. Defaults toTRUE.
Value
A Matrix::dgCMatrix sparse matrix object. Each row corresponds to a different species. Each column corresponds to a different branch. Species that inherit from a given branch are indicated with a one.