Skip to contents

This class is used to represent the constraints used in optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> Constraint

Methods

Public methods

Inherited methods


Constraint$clone()

The objects of this class are cloneable with this method.

Usage

Constraint$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.