This class is used to represent the constraints used in optimization. Only experts should use the fields and methods for this class directly.
Super class
ProjectModifier -> Constraint
This class is used to represent the constraints used in optimization. Only experts should use the fields and methods for this class directly.
ProjectModifier -> Constraint