This class is used to represent the decision variables used in optimization. Only experts should use the fields and methods for this class directly.
Super class
ProjectModifier -> Decision
This class is used to represent the decision variables used in optimization. Only experts should use the fields and methods for this class directly.
ProjectModifier -> Decision