This class is used to represent approaches for multi-objective optimization. Only experts should use the fields and methods for this class directly.
Super class
ProjectModifier -> MultiObjApproach
Methods
MultiObjApproach$calculate()
Perform computations that need to be completed before applying the object.
Arguments
xlistcontaining a compiled multi-objective optimization problem (e.g., generated with internal functionmulti_compile()).ymulti_problem()object.
MultiObjApproach$run()
Solve a multi-objective optimization problem to generate a solution.