This class is used to represent multi-objective project planning
problems. It stores the data (e.g., actions, and features) and
mathematical formulation (e.g., the objective, constraints,
and other design criteria) needed to generate prioritizations.
Most users should use multi_problem() to generate new
multi-objective project problem objects, and the functions distributed
with the package to interact
with them.
Only experts should use the fields and methods for this class directly.
Public fields
problems
list containing ProjectProblem objects.
defaults
list indicating if other fields contain defaults.
approach
MultiObjApproach object
for specifying the multi-objective optimization approach.
solver
Solver object specifying the solver for
generating solutions.
Methods
MultiObjProjectProblem$new()
Create a new multi-objective conservation problem object.
Returns
A new MultiObjProjectProblem object.
MultiObjProjectProblem$print()
Print concise information about the object.
Usage
MultiObjProjectProblem$print()
MultiObjProjectProblem$show()
Display concise information about the object.
Usage
MultiObjProjectProblem$show()
MultiObjProjectProblem$repr()
Generate a character representation of the object.
Usage
MultiObjProjectProblem$repr()
Returns
A character value.
MultiObjProjectProblem$number_of_problems()
Obtain the number of problems.
Usage
MultiObjProjectProblem$number_of_problems()
Returns
An integer value.
MultiObjProjectProblem$number_of_features()
Obtain the number of features.
Usage
MultiObjProjectProblem$number_of_features()
Returns
An integer value.
MultiObjProjectProblem$number_of_actions()
Obtain the number of actions.
Usage
MultiObjProjectProblem$number_of_actions()
Returns
An integer value.
MultiObjProjectProblem$number_of_projects()
Obtain the number of projects.
Usage
MultiObjProjectProblem$number_of_projects()
Returns
An integer value.
MultiObjProjectProblem$problem_names()
Obtain the names of the problems.
Usage
MultiObjProjectProblem$problem_names()
Returns
An character value.
MultiObjProjectProblem$feature_names()
Obtain the names of the features.
Usage
MultiObjProjectProblem$feature_names()
Returns
A list of character vectors.
MultiObjProjectProblem$action_names()
Obtain the names of the actions.
Usage
MultiObjProjectProblem$action_names()
Returns
A character vector.
MultiObjProjectProblem$project_names()
Obtain the names of the projects.
Usage
MultiObjProjectProblem$project_names()
Returns
A list of character vectors.
MultiObjProjectProblem$add_approach()
Create a new object with an approach added to the problem formulation.
Usage
MultiObjProjectProblem$add_approach(x)
Returns
An updated MultiObjProjectProblem object.
MultiObjProjectProblem$add_solver()
Create a new object with a solver added to the problem formulation.
Usage
MultiObjProjectProblem$add_solver(x)
Returns
An updated MultiObjProjectProblem object.
MultiObjProjectProblem$clone()
The objects of this class are cloneable with this method.
Usage
MultiObjProjectProblem$clone(deep = FALSE)
Arguments
deep
Whether to make a deep clone.