This class is used to represent the objective function used in optimization. Only experts should use the fields and methods for this class directly.
Super class
ProjectModifier -> Objective
Public fields
has_targetslogicalvalue indicating if the objective uses targets.has_weightslogicalvalue indicating if the objective uses weights.
Methods
Objective$feature_phylogeny()
Obtain the feature phylogeny.
Returns
A ape::phylo() phylogenetic tree object.
Objective$replace_feature_weights()
Should default feature weights be replaced or multiplied by the new weights?
Objective$evaluate()
Calculate the objective value for a solution.
Arguments
yProjectProblem object.
solutiontibble::tibble()object with solution.