Skip to contents

This super-class is used to represent prototypes that in turn are used to modify a ProjectProblem object. Specifically, the Constraint, Decision, Objective, and Target prototypes inherit from this class. Only experts should use the fields and methods for this class directly.

Public fields

name

character value.

data

list containing data.

internal

list containing internal computed values.

Methods


ProjectModifier$print()

Print information about the object.

Usage

ProjectModifier$print()

Returns

None.


ProjectModifier$show()

Print information about the object.

Usage

ProjectModifier$show()

Returns

None.


ProjectModifier$repr()

Generate a character representation of the object.

Usage

ProjectModifier$repr()

Returns

A character value.


ProjectModifier$get_data()

Get values stored in the data field.

Usage

ProjectModifier$get_data(x)

Arguments

x

character name of data.

Returns

An object. If the data field does not contain an object associated with the argument to x, then a new_waiver() object is returned. Set values stored in the data field. Note that this method will overwrite existing data.


ProjectModifier$set_data()

Usage

ProjectModifier$set_data(x, value)

Arguments

x

character name of data.

value

Object to store.

Returns

Invisible TRUE.


ProjectModifier$get_internal()

Get values stored in the internal field.

Usage

ProjectModifier$get_internal(x)

Arguments

x

character name of data.

Returns

An object. If the internal field does not contain an object associated with the argument to x, then a new_waiver() object is returned.


ProjectModifier$set_internal()

Set values stored in the internal field. Note that this method will overwrite existing data.

Usage

ProjectModifier$set_internal(x, value)

Arguments

x

character name of data.

value

Object to store.

Returns

An object. If the internal field does not contain an object associated with the argument to x, then a new_waiver() object is returned.


ProjectModifier$calculate()

Perform computations that need to be completed before applying the object.

Usage

ProjectModifier$calculate(x, y)

Arguments

Returns

Invisible TRUE.


ProjectModifier$apply()

Update an optimization problem formulation.

Usage

ProjectModifier$apply(x)

Arguments

Returns

Invisible TRUE.


ProjectModifier$clone()

The objects of this class are cloneable with this method.

Usage

ProjectModifier$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.