This super-class is used to represent prototypes that in turn are used to modify a ProjectProblem object. Specifically, the Constraint, Decision, Objective, and Target prototypes inherit from this class. Only experts should use the fields and methods for this class directly.
Public fields
namecharactervalue.datalistcontaining data.internallistcontaining internal computed values.
Methods
ProjectModifier$get_data()
Get values stored in the data field.
Returns
An object. If the data field does not contain an object
associated with the argument to x, then a new_waiver() object is
returned.
Set values stored in the data field. Note that this method will
overwrite existing data.
ProjectModifier$get_internal()
Get values stored in the internal field.
Returns
An object. If the internal field does not contain an object
associated with the argument to x, then a new_waiver() object is
returned.
ProjectModifier$set_internal()
Set values stored in the internal field. Note that this method will
overwrite existing data.
Returns
An object. If the internal field does not contain an object
associated with the argument to x, then a new_waiver() object is
returned.
ProjectModifier$calculate()
Perform computations that need to be completed before applying the object.
Arguments
xnew_optimization_problem()object.yproblem()object.
ProjectModifier$apply()
Update an optimization problem formulation.
Arguments
xnew_optimization_problem()object.