Project problem class
Description
This class is used to represent project prioritization problems. Only experts should use the fields and methods for this class directly.
Public fields
datalistcontaining data (e.g., projects, features).defaultslistindicating if other fields contain defaults.objectiveObjective object used to specify the objective function.
weightsWeight object used to specify the objective function.
decisionsDecision object used to represent the type of decision made on planning units.
targetsTarget object used to represent representation targets for features.
constraintslistobject used to store Constraint objects for the problem.solverSolver object used to specify the process for generating a solution.
Methods
ProjectProblem$new()
Create a new conservation problem object.
Usage
ProjectProblem$new(data = list())ProjectProblem$get_data()
Get values stored in the data field.
Returns
An object. If the data field does not contain an object
associated with the argument to x, then a new_waiver() object is
returned.
ProjectProblem$set_data()
Set values stored in the data field. Note that this method will
overwrite existing data.
ProjectProblem$feature_phylogeny()
Obtain the feature phylogeny.
Returns
A ape::phylo() phylogenetic tree object.
ProjectProblem$project_success_probabilities()
Obtain the probability that each project will succeed if funded.
ProjectProblem$of_matrix()
Obtain information on the outcome for each feature that would be expected if each project funded and is successfully completed.
Returns
A Matrix::dgCMatrix object.
ProjectProblem$pa_matrix()
Obtain information on which actions are associated with each project.
Returns
A Matrix::dgCMatrix object.
ProjectProblem$eof_matrix()
Calculate the expected outcome for each feature assuming that each project is funded and accounting for the possibility that funded projects may fail to be successfully completed.
Returns
A Matrix::dgCMatrix object.
ProjectProblem$add_solver()
Create a new object with a solver added to the problem formulation.
Arguments
xSolver object.
ProjectProblem$add_targets()
Create a new object with targets added to the problem formulation.
Arguments
xTarget object.
ProjectProblem$add_weights()
Create a new object with weights added to the problem formulation.
Arguments
xWeight object.
ProjectProblem$add_objective()
Create a new object with the objective added to the problem formulation.
Arguments
xObjective object.
ProjectProblem$add_decisions()
Create a new object with the decisions added to the problem formulation.
Arguments
xObjective object.
ProjectProblem$add_constraint()
Create a new object with the constraint added to the problem formulation.
Arguments
xConstraint object.