Skip to contents

This class is used to represent solvers for optimization. Only experts should use the fields and methods for this class directly.

Super class

ProjectModifier -> Solver

Public fields

has_pwlobj

logical indicating if solver supports piece-wise linear components in an objective function.

Methods

Inherited methods


Solver$set_start_solution()

Set start solution.

Usage

Solver$set_start_solution(x)

Arguments

x

numeric vector.

Returns

Invisible TRUE.


Solver$remove_start_solution()

Remove start solution.

Usage

Solver$remove_start_solution(x)

Arguments

x

numeric vector.

Returns

Invisible TRUE.


Solver$solve()

Solve an optimization problem.

Usage

Solver$solve(x, ...)

Arguments

x

new_optimization_problem() object.

...

Additional arguments as needed.

Returns

Invisible TRUE.


Solver$clone()

The objects of this class are cloneable with this method.

Usage

Solver$clone(deep = FALSE)

Arguments

deep

Whether to make a deep clone.