This class is used to represent targets for optimization. Only experts should use the fields and methods for this class directly.
Super class
ProjectModifier -> Weight
This class is used to represent targets for optimization. Only experts should use the fields and methods for this class directly.
ProjectModifier -> Weight